
/*-----------------------------------------------------------------
 --             Introduction à OpenGL 
 --                  L3 IUP SI
 --                    M1 IO5                                   
 --              Programme principal                                   
 ------------------------------------------------------------------- 
 */

 
#include <GL/glut.h>
#include "camera.h"
#include "repere.h"
#include "boite.h"


/* Objets de la scene */
static pCamera _camera;
static pRepere _repere;
static pBoite _boite;

/* variables pour les rotations de la scene avec la souris */
static int _xMouse;
static int _yMouse;

static float _transZ = 0;
static float _rotY = 0.;
static float _rotX = 0.;

static float _transZInit;
static float _rotYInit;
static float _rotXInit;

static int _leftClick = 0;
static int _rightClick = 0;

/* fenetre de trace */
static int _windowNumber; 

/* taille de la fenetre */
// static int _wWindow = 600;
// static int _hWindow = 600;
static int _wWindow = 800;
static int _hWindow = 800;

/* position et orientation initiales de la camera */
static Vector _eye = {0.,0.,800.,1.};
static Vector _look = {0.,0.,-1.,1.};
static Vector _up = {0.,1.,-1.,0.};

/* etat d'affichage */
static int _wireFrame = 0;
static int _eclairage = 1;
static int _afficheRepere = 1;

/* type d'eclairage de la lampe GL_LIGHT0 */
static float light0_ambient []= {0.05,0.05,0.05,1.0};
static float light0_diffuse []= {1.,1.,1.,1.0};
static float light0_specular []= {1.,1.,1.,1.0};
static float light0_position []= {0.,0.,0.,1.0};



/* activation de l'eclairage et de la lampe GL_LIGHT0 */
void activeEclairage (void) {

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glLightfv(GL_LIGHT0, GL_AMBIENT, light0_ambient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, light0_diffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, light0_specular);
	_eclairage = 1;
}


/* gestion des evenements clic souris */
void mouse (unsigned int button, int state, int x, int y) {

	if (button==GLUT_LEFT_BUTTON && state==GLUT_DOWN) {
		_leftClick = 1;
		_xMouse=x;
		_yMouse=y;
		_rotYInit = _rotY;
		_rotXInit = _rotX;
	}
	if (button==GLUT_LEFT_BUTTON && state==GLUT_UP) {
		_leftClick = 0;
		
	}
	if (button==GLUT_RIGHT_BUTTON && state==GLUT_DOWN) {
		_rightClick = 1;
		_xMouse=x;
		_yMouse=y;
		_transZInit = _transZ;
	}
	if (button==GLUT_RIGHT_BUTTON && state==GLUT_UP) {
		_rightClick = 0;
	}
}


/* gestion des evenements de deplacement de la souris */
void mouseMotion (int x, int y) {
	if (_leftClick) {
		_rotY = _rotYInit+0.5*(float)(x-_xMouse);
		_rotX = _rotXInit+0.5*(float)(y-_yMouse);	
		glutPostRedisplay();
	}
	if (_rightClick) {
		_transZ = _transZInit+(float)(y-_yMouse);	
		glutPostRedisplay();
	}
}

/* Gestion des evenements clavier */
void key (unsigned char c, int mouseX, int mouseY) {

float pasRotCamX = 0.1;
float pasRotCamZ = 0.1;

  	switch (c) {
	
	/* Touche ESCAPE : quite l'application */
		case 27 : 
	       		glFinish();
			glutDestroyWindow(_windowNumber);
		   	exit (0);
		   	break;
	/* Affichage en filaire / plein */
		case 'w' :
			if (_wireFrame) {
				glPolygonMode (GL_FRONT_AND_BACK, GL_FILL);
				_wireFrame = 0;
			}
			else {
				glPolygonMode (GL_FRONT_AND_BACK, GL_LINE);
				_wireFrame = 1;
			}
			glutPostRedisplay();
		   	break;
	/* Activation / desactivation de l'eclairage */
		case 'e' :
			if (_eclairage) {
				glDisable(GL_LIGHTING);
				_eclairage = 0;
			}
			else activeEclairage();		
			glutPostRedisplay();
			break;
			
	/* active/desactive l'affichage du repere */
		case 'r' :
			if (_afficheRepere) _afficheRepere = 0;
			else _afficheRepere = 1;
			glutPostRedisplay();
			break;
			
	/* boite en plein/fialire */
		case 'b' :
			if (_boite->filaire) _boite->filaire=0;
			else _boite->filaire=1;
			glutPostRedisplay();
			break;
			
	/* regarder vers le haut */
		case 'z' :
			walkRotXCamera (_camera, pasRotCamX); 
			glutPostRedisplay();
			break;
	/* regarder vers le bas */
		case 's' :
			walkRotXCamera (_camera, -pasRotCamX); 
			glutPostRedisplay();
			break;
			
	/* vriller à droite */
		case 'q' :
			walkRotZCamera (_camera, pasRotCamZ); 
			glutPostRedisplay();
			break;
	/* regarder vers le bas */
		case 'd' :
			walkRotZCamera (_camera, -pasRotCamZ); 
			glutPostRedisplay();
			break;
			
	/* retour en position initiale de la camera */
		case 'i' :
			positionCamera (_camera, _eye, _look, _up);
			_rotX=_rotY=0.;
			glutPostRedisplay();
			break;
		
		default :
			break;
	}
}

/* Gestion des interruptions clavier : touches speciales*/
void arrow (int key, int mousex, int mousey) {

float pasRotCamY = 0.1;
float pasAvanceCamZ = 10.;

	switch(key) {
	
	/* regarder vers la droite */
		case GLUT_KEY_RIGHT :
			walkRotYCamera (_camera, -pasRotCamY);
			glutPostRedisplay();
			break;
	/* regarder vers la gauche */
		case GLUT_KEY_LEFT :
			walkRotYCamera (_camera, pasRotCamY); 
			glutPostRedisplay();
			break;			
	/* reculer */
		case GLUT_KEY_DOWN :
			avanceCamera (_camera, -pasAvanceCamZ);
			glutPostRedisplay();
			break;		
	/* avancer */			
		case GLUT_KEY_UP :
			avanceCamera (_camera, pasAvanceCamZ);
			glutPostRedisplay();
			break;
		
		default :
			break;
	}
}

void modifyWindow (int width, int height) {
	_wWindow = width;
	_hWindow = height;
	
	_camera->rapportAspect = ((float)_wWindow)/((float)_hWindow);
}


/* Fonction de rendu */
void display(void) {
	
	/* couleur d'effacement du buffer image : noir */
	glClearColor (0., 0., 0., 0.);
	/* effacement du buffer image et du Z-buffer */
	glClear (GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	/* definition de la fenetre de projection */
	glViewport(0,0,_wWindow,_hWindow);
	/* activation de la matrice de projection */
	glMatrixMode (GL_PROJECTION); 
	/* initialisation de la matrice */
	glLoadIdentity ();
	/* matrice de projection perspective */
	gluPerspective(	_camera->angleOuverture,
			_camera->rapportAspect,
			_camera->planAvant,
			_camera->planArriere);
	/* activation de la matrice de placement des objets */
	glMatrixMode (GL_MODELVIEW);
	/* initialisation de la matrice */
	glLoadIdentity();
	/* Placement de la source de lumière. Elle est attachée à la camera */
	glLightfv(GL_LIGHT0, GL_POSITION, light0_position);
	
	/* Positionne et oriente la camera avec : position
	                                          point visé
	                                          vecteur up(donnant la verticale) */
	gluLookAt (	_camera->o[0], _camera->o[1], _camera->o[2],
			_camera->vise[0], _camera->vise[1], _camera->vise[2],
			_camera->y[0],_camera->y[1],_camera->y[2]);
		
	glPushMatrix();	
		glTranslatef(0.,0.,_transZ);
	/* rotations de la scene */
		glRotatef(_rotY,0.,1.,0.);
		glRotatef(_rotX,1.,0.,0.);
		
/***********************************************/
/*    Debut du tracé des objets de la scène    */
/***********************************************/
	
		/* trace un repère au centre du repère scène */	
		if(_afficheRepere) traceRepere(_repere);
		/* trace de la boite englobante de la scene */	
		glPushMatrix();
			glTranslatef(-256.,-256.,-256.);
			if (_boite->filaire) glDisable (GL_LIGHTING);
			traceBoite(_boite);
			if (_boite->filaire && _eclairage) glEnable (GL_LIGHTING);
		glPopMatrix();
	
	
	
	
/*********************************************/
/*    Fin du tracé des objets de la scène    */
/*********************************************/
	glPopMatrix();
		
	/* echange de buffer dans le double Buffer */
	glutSwapBuffers();
	/* Rafraichissement de la fenetre */
	glFlush ();
}


int main (int argc, char** argv) {

/* creation des objets de la scene */
	_camera=creeCamera(_eye,_look,_up,60.,((float)_wWindow)/((float)_hWindow),0.01,3000.);
	_repere=creeRepere(100.);
	_boite=creeBoite(512.,512.,512.,1,1);

/* Initialisation de l'environement d'affichage glut */
	glutInit (&argc, argv);
	glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	
/* Creation de la fenetre de rendu */
	glutInitWindowSize (_wWindow, _hWindow);
	glutInitWindowPosition (100, 50);
	_windowNumber=glutCreateWindow ("BE modelisation");

/* Activation du Z-buffer */
	glEnable(GL_DEPTH_TEST);
/* Activation de la normalisation automatique des normales */
	glEnable(GL_NORMALIZE);
/* Activation de l'éclairage */
	activeEclairage();
	
/* Gestion des interuptions clavier */
	glutKeyboardFunc (key);
/* Gestion des interruptions clavier : touches speciales*/
	glutSpecialFunc (arrow);
/* Gestion des interruptions souris */
	glutMouseFunc (mouse);
	glutMotionFunc (mouseMotion);
	
/* Affectation de la fonction de rendu dans la fenetre */
	glutDisplayFunc (display);
	
	glutReshapeFunc (modifyWindow);
	
/* Lancement de la boucle d'évènements */
	glutMainLoop();
}
